﻿#region Help:  Introduction to the Script Component
/* The Script Component allows you to perform virtually any operation that can be accomplished in
 * a .Net application within the context of an Integration Services data flow.
 *
 * Expand the other regions which have "Help" prefixes for examples of specific ways to use
 * Integration Services features within this script component. */
#endregion

#region Namespaces
using System;
using System.Data;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;
using Microsoft.SqlServer.Dts.Runtime.Wrapper;
using Microsoft.SqlServer.Dts.Pipeline;
using System.Collections.Generic;
using System.Reflection;
using System.Text;
using System.Diagnostics;
#endregion

/// <summary>
/// This is the class to which to add your code.  Do not change the name, attributes, or parent
/// of this class.
/// </summary>
[Microsoft.SqlServer.Dts.Pipeline.SSISScriptComponentEntryPointAttribute]
public class ScriptMain : UserComponent
{
    static int MaxRows = 100000;
    const int RowsPerBatch = 5000;
    const bool UseRandomIntegerValues = true;
    const int RandomIntMax = 100000;

    static Random random = new Random(5);

    static DateTime DefaultDate = DateTime.Now;

    /// <summary>
    /// This method is called once, before rows begin to be processed in the data flow.
    ///
    /// You can remove this method if you don't need to do anything here.
    /// </summary>
    public override void PreExecute()
    {
        base.PreExecute();

		//
		// Uncomment this line and create a package variable called "MaxRowCount" 
		// to make the number of rows configurable at the package level
		//
        //MaxRows = Variables.MaxRowCount;

        // get the buffer class
        var thisType = this.GetType();
        var fields = thisType.GetFields();
        var bufferMembers = new Dictionary<string, FieldInfo>();

        foreach (var f in fields)
        {
            if (f.FieldType.BaseType == typeof(ScriptBuffer))
            {
                bufferMembers.Add(f.Name, f);
            }
        }

        var outputCollection = this.ComponentMetaData.OutputCollection;
        foreach (IDTSOutput100 output in outputCollection)
        {
            if (output.IsAttached)
            {
                var bufferName = GetBufferName(output.Name);
                var bufferMember = bufferMembers[bufferName];

                var bufferClassType = bufferMember.FieldType;
                var addMethod = bufferClassType.GetMethod("AddRow");

                var bufferInfo = new BufferInfo(bufferMember, addMethod);

                var columns = output.OutputColumnCollection;
                foreach (IDTSOutputColumn100 column in columns)
                {
                    var columnName = GetColumnName(column.Name);
                    var dataType = column.DataType;

                    var member = bufferClassType.GetProperty(columnName);

                    var columnInfo = new ColumnInfo(member, GetDefaultValueForColumn(column, member.PropertyType), dataType);

                    bufferInfo.Columns.Add(columnInfo);
                }

                buffers.Add(bufferInfo);
            }
        }
    }

    static private string GetBufferName(string outputName)
    {
        // TODO: use the right logic
        var sb = new StringBuilder();

        foreach (var c in outputName.ToCharArray())
        {
            if (Char.IsLetterOrDigit(c))
            {
                sb.Append(c);
            }
        }

        sb.Append("Buffer");

        return sb.ToString();
    }

    static private string GetColumnName(string outputColumnName)
    {
        var sb = new StringBuilder();

        foreach (var c in outputColumnName.ToCharArray())
        {
            if (Char.IsLetterOrDigit(c))
            {
                sb.Append(c);
            }
        }

        return sb.ToString();
    }

    static private object GetDefaultValueForColumn(IDTSOutputColumn100 column, Type type)
    {
        object ret;
        switch (column.DataType)
        {
            case DataType.DT_DATE:
            case DataType.DT_DBTIMESTAMP:
            case DataType.DT_BYREF_DBTIMESTAMP2:
            case DataType.DT_DBDATE:
                ret = DefaultDate;
                break;

            case DataType.DT_WSTR:
            case DataType.DT_STR:
            case DataType.DT_NTEXT:
            case DataType.DT_TEXT:
                ret = new string('x', column.Length);
                break;

            case DataType.DT_CY:
            case DataType.DT_DECIMAL:
                ret = new Decimal(1.1);
                break;

            case DataType.DT_BOOL:
                ret = true;
                break;

            default:
                if (type.IsValueType)
                {
                    ret = Activator.CreateInstance(type);
                }
                ret = null;
                break;
        }

        return ret;
    }

    /// <summary>
    /// This method is called after all the rows have passed through this component.
    ///
    /// You can delete this method if you don't need to do anything here.
    /// </summary>
    public override void PostExecute()
    {
        base.PostExecute();
        /*
         * Add your code here
         */
    }

    public override void CreateNewOutputRows()
    {
        int totalCount = 0;
        while (totalCount < MaxRows)
        {
            foreach (var b in buffers)
            {
                for (int i = 0; i < RowsPerBatch; i++)
                {
                    b.AddRow(this);
                }
            }

            totalCount += RowsPerBatch;
        }
    }

    List<BufferInfo> buffers = new List<BufferInfo>();

    class BufferInfo
    {
        public List<ColumnInfo> Columns { get; private set; }

        object buffer = null;
        FieldInfo field;
        MethodInfo addNew;

        public BufferInfo(FieldInfo f, MethodInfo add)
        {
            field = f;
            addNew = add;
            Columns = new List<ColumnInfo>();
        }

        public void AddRow(object component)
        {
            if (buffer == null)
            {
                buffer = field.GetValue(component);
                Debug.Assert(buffer != null, "Buffer object should not be NULL");
            }

            addNew.Invoke(buffer, null);

            foreach (var c in Columns)
            {
                c.Property.SetValue(buffer, c.Value, null);
            }
        }
    }

    class ColumnInfo
    {
        public PropertyInfo Property { get; private set; }
        public object Value
        {
            get
            {
                if (useRandom)
                {
                    return random.Next(RandomIntMax);
                }
                else
                {
                    return cachedValue;
                }
            }
        }

        object cachedValue = null;
        bool useRandom = false;

        public ColumnInfo(PropertyInfo prop, object value, DataType type)
        {
            Property = prop;
            cachedValue = value;

            if (type == DataType.DT_I4 && UseRandomIntegerValues)
            {
                useRandom = true;
            }
        }
    }

}
